_G.EGSScreenshotter = EGSScreenshotter or {}
EGSScreenshotter._mod_path = ModPath
EGSScreenshotter._settings_path = SavePath .. "screenshot_hotkey_settings.json"
EGSScreenshotter._menu_path = EGSScreenshotter._mod_path .. "menu/options.json"

EGSScreenshotter.SCREENSHOT_SFX_PATH = EGSScreenshotter._mod_path .. "assets/screenshot.ogg"
EGSScreenshotter.SCREENSHOT_NAME_FORMAT = "screenshot_%Y%m%d_%H%M%S"
EGSScreenshotter.SCREENSHOT_EXT = "png"
EGSScreenshotter.QKI_URL = "https://pd2mods.z77.fr/quick_keyboard_input.html"

EGSScreenshotter.default_settings = {
	screenshot_sfx_enabled = true,
	screenshot_hide_mouse_enabled = true,
	screenshot_hide_blackmarket_hud = true,
	screenshot_path = SavePath .. "screenshots/"
}
EGSScreenshotter.settings = table.deep_map_copy(EGSScreenshotter.default_settings)

blt.xaudio.setup()

function EGSScreenshotter:LoadSettings()
	local file = io.open(self._settings_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	else
		self:SaveSettings()
	end
end

function EGSScreenshotter:SaveSettings()
	local file = io.open(self._settings_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

function EGSScreenshotter:GetMenuItem(menu_id,item_id)
	local menu = MenuHelper:GetMenu(menu_id)
	if menu then
		-- necessary to get the index, 
		-- as item() does not return the index.
		-- technically could do (#items - (priority+1)) but the last item in the list also has nil priority so.
		-- better to just use the less efficient, more effective way.
		
		local items = menu._items
		if items then 
			for i,item in ipairs(items) do 
				if item:parameter("name") == item_id then
					return item,i
				end
			end
		end
		
		--return menu:item(item_id)
	end
end

-- functional but not used!
function EGSScreenshotter:SetItemTitle(menu_id,item_id,new_str)
	local item,i = self:GetMenuItem(menu_id,item_id)
	if item then
		local gui_node = item:parameter("gui_node")
		if gui_node then
			local row_item = gui_node.row_items[i]
			local gui_text = row_item and row_item.gui_panel
			if gui_text and alive(gui_text) then
				gui_text:set_text(new_str)
				row_item.text = new_str
			end
			item:reload()
			return
		end
	end
end

function EGSScreenshotter:CallbackOnPathChanged(_new_path)
	if _new_path then
		new_path = tostring(_new_path)
		
		-- enforce all forward slashes instead of backslashes
		new_path = string.gsub(new_path,"\\","/")
		
		-- remove redundant slashes
		new_path = string.gsub(new_path,"/+","/")
		
		-- make sure there's anything left of the string
		if string.len(new_path) > 0 then
			local last_char = string.sub(new_path,-1)
			if last_char ~= "/" then
				new_path = new_path .. "/"
			end
			
			if not self.directory_exists(new_path) then
				QuickMenu:new(
					managers.localization:text("menu_egsss_dialog_missing_screenshot_directory_title"),
					managers.localization:text("menu_egsss_dialog_missing_screenshot_directory_desc",
						{
							PATH = new_path
						}
					),
					{
						{
							text = managers.localization:text("dialog_yes"),
							callback = function()
								log(string.format("Creating requested screenshot directory at [%s]",new_path))
								EGSScreenshotter.make_directory(new_path)
							end
						},
						{
							text = managers.localization:text("dialog_no"),
							is_cancel_button = true,
							is_focused_button = true
						}
					},
					true
				)
			end
			self.settings.screenshot_path = new_path
			self:SaveSettings()
			
			managers.localization:add_localized_strings({
				menu_egsss_change_path_title = new_path
			})
			self:SetItemTitle("egsss_options","egsss_change_path",new_path)
			return
		end
	end
	
	error("Screenshot Hotkey: Invalid path string! Please report this to the mod author: " .. tostring(_new_path))
end

function EGSScreenshotter.file_exists(path)
	local _path = Application:nice_path(path, false)
	
	if SystemFS then
		return SystemFS:exists(_path)
	else
		return file.FileExists(_path)
	end
end
function EGSScreenshotter.directory_exists(path)
	local _path = Application:nice_path(path, true)
	
	if SystemFS then
		return SystemFS:exists(_path)
	else
		return file.DirectoryExists(_path)
	end
end
function EGSScreenshotter.make_directory(path)
	local p
	for _, s in pairs(string.split(path, "/")) do
		p = p and p .. "/" .. s  or s
		if not EGSScreenshotter.directory_exists(p) then
			if SystemFS then
				SystemFS:make_dir(p)
			else
				file.CreateDirectory(p)
			end
		end
	end
end

function EGSScreenshotter:CallbackTakeScreenshot()
	local file_exists = self.file_exists
	local directory_exists = self.directory_exists
	
	local SCREENSHOT_PATH_BASE = self.settings.screenshot_path
	local EXT_STR = self.SCREENSHOT_EXT
	
	if directory_exists(SCREENSHOT_PATH_BASE) then
		local screenshot_filename = os.date(self.SCREENSHOT_NAME_FORMAT)
		
		local screenshot_path = Application:nice_path(string.format("%s/%s.%s",SCREENSHOT_PATH_BASE,screenshot_filename,EXT_STR),false)
		if not file_exists(screenshot_path) then
			self:TakeScreenshot(screenshot_path)
		else
			local i = 0
			repeat
				i = i + 1
				screenshot_path = Application:nice_path(string.format("%s/%s_%i.%s",SCREENSHOT_PATH_BASE,screenshot_filename,i,EXT_STR),false)
			until not file_exists(screenshot_path)
			
			self:TakeScreenshot(screenshot_path)
		end
	else
		log(string.format("Error: user screenshot directory does not exist! [%s]",SCREENSHOT_PATH_BASE))
	end
end

function EGSScreenshotter:TakeScreenshot(screenshot_path)
	
	if screenshot_path then
		
		local NON_BLOCKING = true
		-- if false, blocks further execution until screenshot is processed, causing a visible "hitch"
		-- if true, process screenshot "in the background"
		
		local done_cb
		
		local hide_mouse = self.settings.screenshot_hide_mouse_enabled and managers.mouse_pointer._enabled
		if hide_mouse then
			managers.mouse_pointer:disable()
		end
		
		local active_menu = managers.menu:active_menu()
		local hide_blackmarket = self.settings.screenshot_hide_blackmarket_hud and active_menu and active_menu.renderer and active_menu.renderer.ws:visible()
		if hide_blackmarket then
			managers.menu:active_menu().renderer:hide()
		end
		
		done_cb = function()
			log(string.format("Saved screenshot to: %s",screenshot_path))
			
			if hide_mouse then
				managers.mouse_pointer:enable()
			end
			if hide_blackmarket and active_menu and active_menu.renderer then
				active_menu.renderer:show()
			end
			
			if self.settings.screenshot_sfx_enabled then
				XAudio.Source:new(XAudio.Buffer:new(self.SCREENSHOT_SFX_PATH))
				--XAudio.UnitSource:new(XAudio.PLAYER,XAudio.Buffer:new(self.SCREENSHOT_SFX_PATH))
			end
		end
		
		local x,y = 0,0
		local w,h,_ = RenderSettings.resolution:unpack()
		Application:screenshot(screenshot_path,x,y,w,h,NON_BLOCKING,done_cb)
		
		--Application:screenshot(name, x, y, w, h, true, done_callback, resolution_w, resolution_h)
		
	end
end

function EGSScreenshotter:ResetSettings(skip_confirm)
	if skip_confirm then
		self.settings = table.deep_map_copy(self.default_settings)
		
		self:SaveSettings()
		QuickMenu:new(
			managers.localization:text("menu_egsss_dialog_reset_settings_done_title"),
			managers.localization:text("menu_egsss_dialog_reset_settings_done_desc"),
			{
				{
					text = managers.localization:text("menu_ok"),
					is_cancel_button = true,
					is_focused_button = true
				}
			},
			true
		)
	else
		QuickMenu:new(
			managers.localization:text("menu_egsss_dialog_reset_settings_confirm_title"),
			managers.localization:text("menu_egsss_dialog_reset_settings_confirm_desc"),
			{
				{
					text = managers.localization:text("dialog_yes"),
					callback = callback(self,self,"ResetSettings",true)
				},
				{
					text = managers.localization:text("dialog_no"),
					is_cancel_button = true,
					is_focused_button = false
				}
			},
			true
		)
	end
end
		
Hooks:Add("MenuManagerInitialize", "MenuManagerInitialize_egsss", function(menu_manager)
	
	MenuCallbackHandler.callback_egsss_focus_changed = function(this,is_focus)
		if is_focus then
			-- this callback seems to be called before the gui text is refreshed;
			-- so i either need to add some type of delayed callback to reset it accordingly,
			-- or overwrite the localization string for this item
			-- additionally, this still forces the text to uppercase!
			-- annoying!
			
			-- (i hate this i hate this i hate this i hate this)
			local header_item = EGSScreenshotter:GetMenuItem("egsss_options","egsss_header_path")
			if header_item then
				header_item:set_enabled(false)
			end
			
			managers.localization:add_localized_strings({
				menu_egsss_change_path_title = EGSScreenshotter.settings.screenshot_path
			})
			EGSScreenshotter:SetItemTitle("egsss_options","egsss_change_path",EGSScreenshotter.settings.screenshot_path)
		end
	end
	
	MenuCallbackHandler.callback_egsss_enable_sfx = function(this,item)
		EGSScreenshotter.settings.screenshot_sfx_enabled = item:value() == "on"
		EGSScreenshotter:SaveSettings()
	end
	MenuCallbackHandler.callback_egsss_screenshot_hide_mouse = function(this,item)
		EGSScreenshotter.settings.screenshot_hide_mouse_enabled = item:value() == "on"
		EGSScreenshotter:SaveSettings()
	end
	MenuCallbackHandler.callback_egsss_screenshot_hide_blackmarket_hud = function(this,item)
		EGSScreenshotter.settings.screenshot_hide_blackmarket_hud = item:value() == "on"
		EGSScreenshotter:SaveSettings()
	end
	MenuCallbackHandler.callback_egsss_header_path = function(this)
		log("Huh. You shouldn't have been able to press that button. Good job, I guess.")
	end
	MenuCallbackHandler.callback_egsss_change_path = function(this)
		if _G.QuickKeyboardInput then
			QuickKeyboardInput:new(managers.localization:text("menu_egsss_dialog_change_path_title"),managers.localization:text("menu_egsss_dialog_change_path_desc"),{
				default_value = EGSScreenshotter.settings.screenshot_path or "",
				changed_callback = callback(EGSScreenshotter,EGSScreenshotter,"CallbackOnPathChanged")
			},true)
		else
			QuickMenu:new(
				managers.localization:text("menu_egsss_no_qki_title"),
				managers.localization:text("menu_egsss_no_qki_desc",{
						URL = EGSScreenshotter.QKI_URL,
						PATH = EGSScreenshotter._settings_path
					}
				),
				{
					{
						text = managers.localization:text("menu_ok"),
						is_cancel_button = true,
						is_focused_button = true
					}
				},
			true)
		end
	end
	
	MenuCallbackHandler.callback_egsss_take_screenshot = function(this)
		EGSScreenshotter:CallbackTakeScreenshot()
	end
	
	MenuCallbackHandler.callback_egsss_reset_settings = function(this)
		EGSScreenshotter:ResetSettings(false)
	end
	
	EGSScreenshotter:LoadSettings()
	if not EGSScreenshotter.directory_exists(EGSScreenshotter.settings.screenshot_path) then
		log("Screenshot directory not found. Creating...")
		EGSScreenshotter.make_directory(EGSScreenshotter.settings.screenshot_path)
	end
	
	MenuHelper:LoadFromJsonFile(EGSScreenshotter._menu_path, EGSScreenshotter, EGSScreenshotter.settings)
end)